//------------------------------------------------------------------------------
//
//  npengine.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npengine.h"

#include "../npdata.h"
#include "../data/npplot.h"



void UpdatePlot (NPplotPtr plot);
void UpdatePlotDefault (NPplotPtr plot);
void UpdatePlotCamera (NPplotPtr plot);
void UpdatePlotSurface (NPplotPtr plot);
void UpdatePlotPoints (NPplotPtr plot);
void UpdatePlotPin (NPplotPtr plot);
void UpdatePlotVideo (NPplotPtr plot);
void UpdatePlotGrid (NPplotPtr plot);


//------------------------------------------------------------------------------
void npInitEngine (void* dataRef)
{

}


//------------------------------------------------------------------------------
void npCloseEngine (void* dataRef)
{

}


// iterate through plots update translation rotation and scale
//------------------------------------------------------------------------------
void npEngineUpdate (void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;

	for (i=0; i < data->map.plotCount; i++)
		UpdatePlot (data->map.plot[i]);
}


//------------------------------------------------------------------------------
void UpdatePlot (NPplotPtr plot)
{
	switch (plot->type)
	{
		case kPlotDefault : UpdatePlotDefault (plot); break;
		case kPlotCamera : UpdatePlotCamera (plot); break;
		case kPlotSurface : UpdatePlotSurface (plot); break;
		case kPlotPoints : UpdatePlotPoints (plot); break;
		case kPlotPin : UpdatePlotPin (plot); break;
		case kPlotVideo : UpdatePlotVideo (plot); break;
		case kPlotGrid : UpdatePlotGrid (plot); break;
	}
}


//------------------------------------------------------------------------------
void UpdatePlotDefault (NPplotPtr plot)
{
	GLfloat sinX, cosX, sinZ, cosZ;

	// update rotation about XYZ axis and keep inbounds of 2 PI
	plot->rotateRad.x += plot->rotateRate.x;
	if (plot->rotateRad.x >= kPos2PI)
		plot->rotateRad.x -= kPos2PI;
	if (plot->rotateRad.x <= kNeg2PI)
		plot->rotateRad.x += -kPos2PI;

	plot->rotateRad.y += plot->rotateRate.y;
	if (plot->rotateRad.y >= kPos2PI)
		plot->rotateRad.y -= kPos2PI;
	if (plot->rotateRad.y <= kNeg2PI)
		plot->rotateRad.y += kPos2PI;

	plot->rotateRad.z += plot->rotateRate.z;
	if (plot->rotateRad.z >= kPos2PI)
		plot->rotateRad.z -= kPos2PI;
	if (plot->rotateRad.z <= kNeg2PI)
		plot->rotateRad.z += kPos2PI;

	//calculate the rotation vector		// optimize with lookup tables, zz
	sinX = (GLfloat)sin(plot->rotateRad.x);
	cosX = (GLfloat)cos(plot->rotateRad.x);
	sinZ = (GLfloat)sin(plot->rotateRad.z);
	cosZ = (GLfloat)cos(plot->rotateRad.z);

	plot->rotate.x = sinZ * cosX;
	plot->rotate.y = cosZ * cosX;
	plot->rotate.z = -sinX;

	//update XYZ scale
	plot->scale.x += plot->scaleRate.x;
	plot->scale.y += plot->scaleRate.y;
	plot->scale.z += plot->scaleRate.z;
	
	//update the translateVect for selected objects
	plot->translateVect.x = sinZ * plot->translateRate.y
						  + cosZ * plot->translateRate.x;

	plot->translateVect.y = cosZ * plot->translateRate.y
						  - sinZ * plot->translateRate.x;

	//z ignores the angles just move in global coordinates, 0.5 to reduce speed
	plot->translateVect.z = plot->translateRate.z * 0.5f;

	//update the position by adding the vector to existing position
	plot->translate.x += plot->translateVect.x;
	plot->translate.y += plot->translateVect.y;
	plot->translate.z += plot->translateVect.z;

	//limit from going below the surface
	if (plot->triggerOnLow.z)
		if (plot->translate.z < plot->triggerLevelLow.z)											// debug, zz
			plot->translate.z = plot->triggerLevelLow.z;
}


//------------------------------------------------------------------------------
void UpdatePlotCamera (NPplotPtr plot)
{
	GLfloat sinX, cosX, sinZ, cosZ;

	// update rotation about XYZ axis and keep inbounds of 2 PI
	plot->rotateRad.x += plot->rotateRate.x;
	if (plot->rotateRad.x >= kPos2PI / 4.0f)	// limit to +/- 90 deg
		plot->rotateRad.x = kPos2PI / 4.0f;
	if (plot->rotateRad.x <= kNeg2PI / 4.0f)
		plot->rotateRad.x = -kPos2PI / 4.0f;

	plot->rotateRad.y += plot->rotateRate.y;
	if (plot->rotateRad.y >= kPos2PI)
		plot->rotateRad.y -= kPos2PI;
	if (plot->rotateRad.y <= kNeg2PI)
		plot->rotateRad.y += kPos2PI;

	plot->rotateRad.z += plot->rotateRate.z;
	if (plot->rotateRad.z >= kPos2PI)
		plot->rotateRad.z -= kPos2PI;
	if (plot->rotateRad.z <= kNeg2PI)
		plot->rotateRad.z += kPos2PI;

	//calculate the rotation vector		// optimize with lookup tables, zz
	sinX = (GLfloat)sin(plot->rotateRad.x);
	cosX = (GLfloat)cos(plot->rotateRad.x);
	sinZ = (GLfloat)sin(plot->rotateRad.z);
	cosZ = (GLfloat)cos(plot->rotateRad.z);

	plot->rotate.x = sinZ * cosX;
	plot->rotate.y = cosZ * cosX;
	plot->rotate.z = -sinX;

	//update XYZ scale
	plot->scale.x += plot->scaleRate.x;
	plot->scale.y += plot->scaleRate.y;
	plot->scale.z += plot->scaleRate.z;
	
	//update the translateVect for selected objects
	plot->translateVect.x = sinZ * plot->translateRate.y
						  + cosZ * plot->translateRate.x;

	plot->translateVect.y = cosZ * plot->translateRate.y
						  - sinZ * plot->translateRate.x;

	//z ignores the angles just move in global coordinates, 0.5 to reduce speed
	plot->translateVect.z = plot->translateRate.z * 0.5f;

	//update the position by adding the vector to existing position
	plot->translate.x += plot->translateVect.x;
	plot->translate.y += plot->translateVect.y;
	plot->translate.z += plot->translateVect.z;

	//limit from going below the surface
	if (plot->triggerOnLow.z)
		if (plot->translate.z < plot->triggerLevelLow.z)											// debug, zz
			plot->translate.z = plot->triggerLevelLow.z;
}


//------------------------------------------------------------------------------
void UpdatePlotSurface (NPplotPtr plot)
{
	UpdatePlotDefault (plot);
}


//------------------------------------------------------------------------------
void UpdatePlotPoints (NPplotPtr plot)
{
	UpdatePlotDefault (plot);
}


//------------------------------------------------------------------------------
void UpdatePlotPin (NPplotPtr plot)
{
	UpdatePlotDefault (plot);
}


//------------------------------------------------------------------------------
void UpdatePlotVideo (NPplotPtr plot)
{
	UpdatePlotDefault (plot);
}


//------------------------------------------------------------------------------
void UpdatePlotGrid (NPplotPtr plot)
{
	UpdatePlotDefault (plot);
}


