//------------------------------------------------------------------------------
//
//  npgldraw.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npgldraw.h"

#include <stdlib.h>

#include "../npglut.h"
#include "../../data/npplot.h"
#include "../../npctrl.h"


#define kTorusScreenSizeCount 4
#define kTorusThicknessCount 4

static GLint pin_display_list;


void DrawDefault (NPplotPtr plot);
void DrawCamera (NPplotPtr plot);
void DrawSurface (NPplotPtr plot);
void DrawPoints (NPplotPtr plot);
void DrawPin (NPplotPtr plot);
void DrawVideo (NPplotPtr plot);
void DrawGrid (NPplotPtr plot);


struct NPtorus {
	GLfloatXYZPtr	vArray;
	GLint			vArraySize;
	GLint*			quadArray;
	GLint			quadArraySize;
	int				slices;
	int				segments;
	GLfloat			thickness;
};
typedef struct NPtorus NPtorus;
typedef struct NPtorus * NPtorusPtr;

struct NPcircle {
	GLfloatXYPtr	vArray;
	int				segments;
	GLfloat			radius;
};
typedef struct NPcircle NPcircle;
typedef struct NPcircle * NPcirclePtr;


void InitTorus (void* dataRef);

GLuint CreatePinDL();
void DrawPinDL();


void npDrawTorus (GLfloat radius, GLfloat thickness);

void CreateTorus (NPtorusPtr torus);
void CreateCircle (NPcirclePtr circle);
void DeleteCircle (NPcirclePtr circle);



//------------------------------------------------------------------------------
void npInitGLDraw(void* dataRef)
{
//	InitTorus (dataRef);

	pin_display_list = CreatePinDL();
}


// creates a set of toroids and loads them into GPU memory
//------------------------------------------------------------------------------
void InitTorus (void* dataRef)
{
	int i = 0, j = 0;
	NPtorusPtr torus = NULL;

	torus = (NPtorusPtr) malloc (sizeof(NPtorus));


	// 5x5, 7x5, 12x6, 16x6, 24x8, 32x10, 48x12, 64x16, 96x24, 128x32, 196x48
	for ( i = 0; i < kTorusScreenSizeCount; i++)
		for ( j = 0; j < kTorusThicknessCount; j++)
		{
			switch (i)
			{
				case 0 : torus->slices = 5; torus->segments = 5; break;
				case 1 : torus->slices = 5; torus->segments = 7; break;
				case 2 : torus->slices = 6; torus->segments = 12; break;
				case 3 : torus->slices = 6; torus->segments = 16; break;
				case 4 : torus->slices = 8; torus->segments = 24; break;
				case 5 : torus->slices = 10; torus->segments = 32; break;
				case 6 : torus->slices = 12; torus->segments = 48; break;
				case 7 : torus->slices = 16; torus->segments = 64; break;
				case 8 : torus->slices = 24; torus->segments = 96; break;
				case 9 : torus->slices = 32; torus->segments = 128; break;
				case 10 : torus->slices = 48; torus->segments = 196; break;
				case 11 : torus->slices = 64; torus->segments = 256; break;
			}
			
			torus->thickness =	(GLfloat) j * 100.0f / 
								(GLfloat) (kTorusThicknessCount - 1);

			CreateTorus (torus);
			// load torus to GPU and deallocate memory
			// we could save time by allocating enough memory for all torus's at once
			// or enough for largest torus then de-allocating all at once
		}
}


// creates a single torus of specified slices, stacks and thickness
//------------------------------------------------------------------------------
void CreateTorus (NPtorusPtr torus)
{
/*	int i=0, j=0;

	NPcirclePtr ring = (NPcirclePtr) malloc (sizeof(NPcircle));
	NPcirclePtr hoops = (NPcirclePtr) malloc (torus->slices * sizeof(NPcircle));

	torus->vArraySize = torus->slices * torus->segments;
	torus->vArray = (GLfloatXYZPtr) malloc (torus->vArraySize * sizeof(GLfloatXYZ));

	torus->quadArraySize = torus->vArraySize * 4;
	torus->quadArray = (GLint*) malloc (torus->quadArraySize * sizeof(GLint));

	// create our tube ring
	ring->radius = torus->thickness;		// 1.0 is no hole in the donut
	ring->segments = torus->slices;
	CreateCircle (ring);

	// create our hoops, the length segments of the tube
	for (i=0; i < torus->slices; i++)
	{
		hoops[i].radius = 1.0f + ring->vArray[i].x;
		hoops[i].segments = torus->segments;
		CreateCircle (&hoops[i]);
	}

	// build the vertex array for the torus
	for (i=0; i < torus->segments; i++)
		for (j=0; j < torus->slices; j++)
		{
			torus->vArray[i*torus->slices + j].x = hoops[j].vArray[i].x;		//the hoop provides x and y
			torus->vArray[i*torus->slices + j].y = hoops[j].vArray[i].y;
			torus->vArray[i*torus->slices + j].z = ring->vArray[j].y;		//the ring provides the z
		}

	// build the the torus polygon array
	for (i=0; i < torus->segments; i++)
		for (j=0; j < torus->slices; j++)
		{	
			if ( i == (torus->segments - 1) )		// the last torus segment
			{
				if ( j == (torus->slices - 1) )		// the last polygon
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices);
					torus->quadArray[i][j][2] = 0;
					torus->quadArray[i][j][3] = j;
				}
				else
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices) + (j + 1);
					torus->quadArray[i][j][2] = (j + 1);
					torus->quadArray[i][j][3] = j;
				}
			}
			else
			{
				if ( j == (torus->slices - 1) )		// the last polygon in a segment
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices);
					torus->quadArray[i][j][2] = ((i + 1) * torus->slices);
					torus->quadArray[i][j][3] = ((i + 1) * torus->slices) + j;
				}
				else
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices) + (j + 1);
					torus->quadArray[i][j][2] = ((i + 1) * torus->slices) + (j + 1);
					torus->quadArray[i][j][3] = ((i + 1) * torus->slices) + j;
				}
			}

	DeleteCircle (ring);
	for (i=0; i < torus->slices; i++)
		DeleteCircle (&hoops[i]);
*/
}


// creates a circle of specified number of points and radius
// centered on the origin (0,0) starting at 12 o'clock going CW
// really should be part of GL, glCircle!, lol
//------------------------------------------------------------------------------
void CreateCircle (NPcirclePtr circle)
{
/*	int i = 0;
	GLfloat angle = 0.0, deltaAngle = 0.0f;

	circle->vArray = (GLfloatXY) malloc (cirlce->segments * sizeof(GLfloatXY));


	if (circle->segments)
		deltaAngle = (2.0 * kPI) / circle->segments;
	
	// go around the circle, 
	for (angle = 0.0f; angle <= 1.99999f * kPI; angle += deltaAngle)
	{
//		circle->vArray[i]->x = circle->radius * (GLfloat)sin(angle);
//		circle->vArray[i].y = circle->radius * (GLfloat)cos(angle);
		i++;
	}
*/
}


//------------------------------------------------------------------------------
void DeleteCircle (NPcirclePtr circle)
{
	free (circle->vArray);
	free (circle);
}


// draws a torus of specified size and thickness, stacks and slices determined
// by size in pixels on screen calculated by screen size and z distance
// radius from center of the donut to the center of the tube, thickness 0-100%
//------------------------------------------------------------------------------
void npDrawTorus (GLfloat radius, GLfloat thickness)
{
	int torusID = 0;

	// torusID = screenSize + (thickness / numberOfSizes);

	glPushMatrix();
		glScalef(radius, radius, radius);
		glCallList(torusID);
	glPopMatrix();
}



//------------------------------------------------------------------------------
void DrawPinDL() 
{
	glPushMatrix();

	glColor3f(0.0f, 1.0f, 0.0f);

// Draw Pin
	glTranslatef (0.0f, 0.0f, 5.0f);
	glPushMatrix();
		glRotatef (180.0f, 0.0f, 1.0f, 0.0f);
		npGLWireCone (0.5f, 5.0f, 10, 1);
	glPopMatrix();

// Draw Discs
	glPushMatrix();
		glScalef (1.0f, 1.0f, 0.1f);
		npGLWireSphere (1.75f, 12, 8);
	glPopMatrix();

	glPushMatrix();
		glTranslatef (0.0f, 1.75f, 0.0f);
		glScalef (0.1f, 1.0f, 1.0f);
		glRotatef (90.0f, 0.0f, 0.0f, 1.0f);
		npGLWireSphere (0.75f, 9, 9);
	glPopMatrix();
	
	glColor3f (1.0f, 0.0f, 0.0f);
	glTranslatef (-1.75f, 0.0f, 0.0f);
	glScalef (1.0f, 0.1f, 1.0f);
	npGLWireSphere (0.75f, 9, 9);

	glPopMatrix();
}


//------------------------------------------------------------------------------
GLuint CreatePinDL()
{
	GLuint pinDL;

	// Create the id for the list
	pinDL = glGenLists(1);

	// start list
	glNewList (pinDL,GL_COMPILE);

	// call the function that contains the rendering commands
	DrawPinDL();

	// endList
	glEndList();

	return (pinDL);
}

										// place holder until full plot drawing, zz debug
//------------------------------------------------------------------------------
void DrawTemp()
{
	int i = 0;
	int j = 0;

	glColor3f (0.3f, 0.25f, 0.0f);
	glBegin (GL_QUADS);
		glVertex3f (-100.0f, -100.0f, 0.0f);
		glVertex3f (-100.0f,  100.0f, 0.0f);
		glVertex3f ( 100.0f,  100.0f, 0.0f);
		glVertex3f ( 100.0f, -100.0f, 0.0f);
	glEnd();	
	
	// draw 6x6 grid of pins

	for(i = -3; i < 3; i++)
		for(j=-3; j < 3; j++) {
			glPushMatrix();
			glTranslatef (i * 10.0f, j * 10.0f, 0.0f);
			glCallList (pin_display_list);
			glPopMatrix();
		}
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void npDrawPlots (void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;
	NPplotPtr plot = NULL;

	for (i=0; i < data->map.plotCount; i++)
	{
		plot = data->map.plot[i];

		switch (plot->type)
		{
			case kPlotDefault : DrawDefault (plot); break;
			case kPlotCamera : DrawCamera (plot); break;
			case kPlotSurface : DrawSurface (plot); break;
			case kPlotPoints : DrawPoints (plot); break;
			case kPlotPin : DrawPin (plot); break;
			case kPlotVideo : DrawVideo (plot); break;
			case kPlotGrid : DrawGrid (plot); break;
		}
	}
	DrawTemp();
}


//------------------------------------------------------------------------------
void DrawDefault (NPplotPtr plot)
{

}


// draw an icon at the location of the camera in the scene
//------------------------------------------------------------------------------
void DrawCamera (NPplotPtr plot)
{
	// draw icon of camera in the scene, but not the current viewport camera
}


//------------------------------------------------------------------------------
void DrawSurface (NPplotPtr plot)
{
	// draw ground

	glColor3f(0.3f, 0.25f, 0.0f);
	glBegin(GL_QUADS);
		glVertex3f(-100.0f, -100.0f, 0.0f);
		glVertex3f(-100.0f,  100.0f, 0.0f);
		glVertex3f( 100.0f,  100.0f, 0.0f);
		glVertex3f( 100.0f, -100.0f, 0.0f);
	glEnd();
}


//------------------------------------------------------------------------------
void DrawPoints (NPplotPtr plot)
{

}


//------------------------------------------------------------------------------
void DrawPin (NPplotPtr plot)
{
	glPushMatrix();

	// position the plot,			may need to re-order transformations, debug, zz
	glScalef (plot->scale.x, plot->scale.y, plot->scale.z);
	glRotatef (plot->rotate.angle, plot->rotate.x, plot->rotate.y, plot->rotate.z);
	glTranslatef (plot->translate.x, plot->translate.y, plot->translate.z);
	
	glCallList (pin_display_list);

	glPopMatrix();
}


//------------------------------------------------------------------------------
void DrawVideo (NPplotPtr plot)
{

}


//------------------------------------------------------------------------------
void DrawGrid (NPplotPtr plot)
{

}


// draw 2D overlays including grids, text, messages, highlight objects, etc...
//------------------------------------------------------------------------------
void npDrawHUD (void* dataRef)
{

}



